@@ -1,5 +1,5 @@
 from datetime import datetime, timedelta
-from typing import Sequence, Callable, Dict, Any, List
+from typing import Sequence, Callable, Dict, Any, List, Union
 
 import numpy as np
 from tqdm import tqdm
@@ -37,11 +37,11 @@ class Policy:
         The history of ``train_losses`` and ``metrics`` from the entire ``epoch`` is provided as additional information.
         """
 
-    def epoch_finished(self, epoch: int, train_losses: Sequence, metrics: dict = None):
+    def epoch_finished(self, epoch: int, train_losses: Sequence, metrics: dict = None, policies: dict = None):
         """
         Update the policy after an epoch is finished. The epochs numeration starts at zero.
 
-        The history of ``train_losses`` and ``metrics`` from the entire ``epoch`` is provided as additional information.
+        The history of ``train_losses`` and ``metrics`` and ``policies`` from the entire ``epoch`` is provided as additional information.
         """
 
 
@@ -57,6 +57,7 @@ class ValuePolicy(Policy, metaclass=ABCAttributesMeta):
 
     def __init__(self, initial):
         super().__init__()
+        # TODO: turn self.value into a property
         self.value = initial
 
 
@@ -68,6 +69,8 @@ class DecreasingOnPlateau(ValuePolicy):
     """
     Policy that traces average train loss and if it didn't decrease according to ``atol``
     or ``rtol`` for ``patience`` epochs, multiply `value` by ``multiplier``.
+    ``atol`` :- absolute tolerance for detecting change in training loss value.
+    ``rtol`` :- relative tolerance for detecting change in training loss value.
     """
 
     def __init__(self, *, initial: float, multiplier: float, patience: int, rtol, atol):
@@ -81,19 +84,18 @@ class DecreasingOnPlateau(ValuePolicy):
         self.margin_loss = np.inf
 
     def get_margin_loss(self, loss):
-        return max([loss * (1 - self.rtol), loss - self.atol])
+        return max(loss - self.atol, loss * (1 - self.rtol))
 
-    def epoch_finished(self, epoch: int, train_losses: Sequence, **kwargs):
+    def epoch_finished(self, epoch: int, train_losses: Sequence, **kwargs) -> None:
         loss = np.mean(train_losses)
         if loss < self.margin_loss:
             self.margin_loss = self.get_margin_loss(loss)
             self.epochs_waited = 0
-        else:
-            self.epochs_waited += 1
-
-            if self.epochs_waited > self.patience:
-                self.value *= self.lr_dec_mul
-                self.epochs_waited = 0
+            return
+        self.epochs_waited += 1
+        if self.epochs_waited > self.patience:
+            self.value *= self.lr_dec_mul
+            self.epochs_waited = 0
 
 
 class Exponential(ValuePolicy):
@@ -102,18 +104,21 @@ class Exponential(ValuePolicy):
     If ``floordiv`` is False - the `value` will be changed continuously.
     """
 
-    def __init__(self, initial: float, multiplier: float, step_length: int = 1, floordiv: bool = True):
+    def __init__(self, initial: float, multiplier: float, step_length: int = 1, floordiv: bool = True,
+                 min_value: float = -np.inf, max_value: float = np.inf):
         super().__init__(initial)
         self.multiplier = multiplier
         self.initial = initial
         self.step_length = step_length
         self.floordiv = floordiv
+        self.min_value = min_value
+        self.max_value = max_value
 
     def epoch_started(self, epoch: int):
         power = epoch / self.step_length
         if self.floordiv:
             power = np.floor(power)
-        self.value = self.initial * self.multiplier ** power
+        self.value = np.clip(self.initial * self.multiplier ** power, self.min_value, self.max_value)
 
 
 class Schedule(ValuePolicy):
@@ -182,7 +187,7 @@ class LossStop(Policy):
         self.min_loss = np.inf
         self.max_ratio = max_ratio
 
-    def epoch_finished(self, epoch, *, train_losses: Sequence[float] = None, metrics: dict = None):
+    def epoch_finished(self, epoch, *, train_losses: Sequence[float] = None, metrics: dict = None, **kwargs):
         loss = np.mean(train_losses)
         self.min_loss = min(self.min_loss, loss)
         if loss > self.max_ratio * self.min_loss:
@@ -251,7 +256,7 @@ class TimeProfiler(Policy):
     def train_step_finished(self, epoch: int, iteration: int, loss: Any):
         self.stamps.append(datetime.now())
 
-    def epoch_finished(self, epoch: int, train_losses: Sequence, metrics: dict = None):
+    def epoch_finished(self, epoch: int, train_losses: Sequence, metrics: dict = None, **kwargs):
         self._display(epoch)
 
     # this policy is stateless
@@ -280,3 +285,24 @@ class TQDM(Policy):
     def validation_started(self, epoch: int, train_losses: Sequence):
         self.bar.close()
         self.bar = None
+
+
+class LoggerPolicy(Policy):
+    def __init__(self, logger):
+        self.logger = logger
+
+    def train_step_finished(self, epoch: int, iteration: int, loss: Union[dict, float]):
+        if not isinstance(loss, dict):
+            loss = {'train/loss_step': loss}
+        else:
+            loss = {f'train/loss_step/{name}': v for name, v in loss.items()}
+
+        for name, value in loss.items():
+            self.logger.value(name, value)
+
+    def epoch_finished(self, epoch: int, train_losses: Sequence, metrics: dict = None, policies: dict = None):
+        self.logger.train(train_losses, epoch)
+        if metrics is not None:
+            self.logger.metrics(metrics, epoch)
+        if policies is not None:
+            self.logger.policies(policies, epoch)
